@@ -3,7 +3,7 @@ package POE::Component::Server::TCP;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '1.294_512'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.020'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw(carp croak);
 use Socket qw(INADDR_ANY inet_ntoa inet_aton AF_INET AF_UNIX PF_UNIX);
@@ -46,8 +46,7 @@ sub new {
 
   foreach (
     qw(
-      Acceptor Error ClientInput
-      ClientPreConnect ClientConnected ClientDisconnected
+      Acceptor Error ClientInput ClientConnected ClientDisconnected
       ClientError ClientFlushed
       ClientLow ClientHigh
     )
@@ -88,7 +87,6 @@ sub new {
     }
   }
 
-  my $client_pre_connect  = delete $param{ClientPreConnect};
   my $client_connected    = delete $param{ClientConnected};
   my $client_disconnected = delete $param{ClientDisconnected};
   my $client_error        = delete $param{ClientError};
@@ -128,6 +126,7 @@ sub new {
   $address = INADDR_ANY unless defined $address;
 
   $error_callback = \&_default_server_error unless defined $error_callback;
+  $server_started = sub {} unless ref($server_started) eq 'CODE';
 
   $session_type = 'POE::Session' unless defined $session_type;
   if (defined($session_params) && ref($session_params)) {
@@ -140,6 +139,9 @@ sub new {
 
   if (defined $client_input) {
     $client_error  = \&_default_client_error unless defined $client_error;
+    $client_connected    = sub {} unless defined $client_connected;
+    $client_disconnected = sub {} unless defined $client_disconnected;
+    $client_flushed      = sub {} unless defined $client_flushed;
     $client_args         = []     unless defined $client_args;
 
     # Extra states.
@@ -239,6 +241,7 @@ sub new {
       $accept_callback = sub {
         my ($socket, $remote_addr, $remote_port) = @_[ARG0, ARG1, ARG2];
 
+
         $session_type->create(
           @$session_params,
           inline_states => {
@@ -264,25 +267,14 @@ sub new {
                 $heap->{remote_ip} = inet_ntoa($remote_addr);
               }
               else {
-                $heap->{remote_ip} = (
-                  Socket::GetAddrInfo::getaddrinfo($remote_addr)
-                )[1];
+                $heap->{remote_ip} =
+                  Socket6::inet_ntop($domain, $remote_addr);
               }
 
               $heap->{remote_port} = $remote_port;
 
-              my $socket = $_[ARG0];
-              if ($client_pre_connect) {
-                $socket = $client_pre_connect->(@_);
-                unless (fileno($socket)) {
-                  # TODO - Error callback?  Disconnected callback?
-                  # TODO - Should we do this before starting the child?
-                  return;
-                }
-              }
-
               $heap->{client} = POE::Wheel::ReadWrite->new(
-                Handle       => $socket,
+                Handle       => $_[ARG0],
                 Driver       => POE::Driver::SysRW->new(),
                 _get_filters(
                   $client_filter,
@@ -309,7 +301,7 @@ sub new {
               # into @_[ARG0..].  There are only 2 parameters.
               splice(@_, ARG0, 2, @{$_[ARG1]});
 
-              $client_connected and $client_connected->(@_);
+              $client_connected->(@_);
             },
             tcp_server_got_high => $high_event,
             tcp_server_got_low => $low_event,
@@ -337,10 +329,10 @@ sub new {
             tcp_server_got_flush => sub {
               my $heap = $_[HEAP];
               DEBUG and warn "$$: $alias child Flush";
-              $client_flushed and $client_flushed->(@_);
+              $client_flushed->(@_);
               if ($heap->{shutdown}) {
                 DEBUG and warn "$$: $alias child Flush, callback";
-                $client_disconnected and $client_disconnected->(@_);
+                $client_disconnected->(@_);
                 delete $heap->{client};
               }
             },
@@ -354,7 +346,7 @@ sub new {
                   not $heap->{client}->get_driver_out_octets()
                 ) {
                   DEBUG and warn "$$: $alias child Shutdown, callback";
-                  $client_disconnected and $client_disconnected->(@_);
+                  $client_disconnected->(@_);
                   delete $heap->{client};
                 }
               }
@@ -447,7 +439,7 @@ sub new {
           SuccessEvent => 'tcp_server_got_connection',
           FailureEvent => 'tcp_server_got_error',
         );
-        $server_started and $server_started->(@_);
+        $server_started->(@_);
       },
       # Catch an error.
       tcp_server_got_error => $error_callback,
@@ -585,7 +577,7 @@ sub _test_filter {
     if (!$eval and $@) {
         carp(
           "Failed to load [$filter]\n" .
-          "Reason $@\nUsing default POE::Filter::Line "
+          "Reason $@\nUsing defualt POE::Filter::Line "
         );
         return 0;
     }
@@ -699,7 +691,7 @@ that manages such a hash.
   }
 
 The component's C<Started> callback is invoked at the end of the
-master session's start-up routine.  The @_[ARG0..$#_] parameters are
+master session's startup routine.  The @_[ARG0..$#_] parameters are
 set to a copy of the values in the server's C<ListenerArgs>
 constructor parameter.  The other parameters are standard for
 POE::Session's _start handlers.
@@ -707,8 +699,8 @@ POE::Session's _start handlers.
 The component's C<Error> callback is invoked when the server has a
 problem listening for connections.  C<Error> may also be called if the
 component's default acceptor has trouble accepting a connection.
-C<Error> receives the usual ones for L<POE::Wheel::SocketFactory/FailureEvent> and
-L<POE::Wheel::ReadWrite/ErrorEvent>.
+C<Error> receives the usual ones for POE::Wheel::SocketFactory and
+POE::Wheel::ReadWrite L<ErrorEvent>.
 
 =head2 Default Child Connection Sessions
 
@@ -731,42 +723,18 @@ The component's C<ClientInput> callback defines how child sessions
 will handle input from their clients.  Its parameters are that of
 POE::Wheel::ReadWrite's C<InputEvent>.
 
-As mentioned C<ClientConnected> is called at the end of the child
-session's C<_start> routine.  The C<ClientConneted> callback receives
-the same parameters as the client session's _start does.  The arrayref
-passed to the constructor's C<Args> parameter is flattened and
-included in C<ClientConnected>'s parameters as @_[ARG0..$#_].
+As mentioned
+C<ClientConnected> is called at the end of the child session's
+C<_start> routine.  The C<ClientConneted> callback receives the same
+parameters as the client session's _start does.  The arrayref passed
+to the constructor's C<Args> parameter is flattened and included in
+C<ClientConnected>'s parameters as @_[ARG0..$#_].
 
   sub handle_client_connected {
     my @constructor_args = @_[ARG0..$#_];
     ...
   }
 
-C<ClientPreConnect> is called before C<ClientConnected>, and it has
-different parameters: $_[ARG0] contains a copy of the client socket
-before it's given to POE::Wheel::ReadWrite for management.  Most HEAP
-members are set, except of course $_[HEAP]{client}, because the
-POE::Wheel::ReadWrite has not yet been created yet.
-C<ClientPreConnect> may enable SSL on the socket, using
-POE::Component::SSLify.  C<ClientPreConnect> must return a valid
-socket to complete the connection; the client will be disconnected if
-anything else is returned.
-
-  sub handle_client_pre_connect {
-
-    # Make sure the remote address and port are valid.
-    return undef unless validate(
-      $_[HEAP]{remote_ip}, $_[HEAP]{remote_port}
-    );
-
-    # SSLify the socket, which is in $_[ARG0].
-    my $socket = eval { Server_SSLify($_[ARG0]) };
-    return undef if $@;
-
-    # Return the SSL-ified socket.
-    return $socket;
-  }
-
 C<ClientDisconnected> is called when the client has disconnected,
 either because the remote socket endpoint has closed or the local
 endpoint has been closed by the server.  This doesn't mean the
@@ -814,7 +782,7 @@ C<Alias> parameter.
 
 POE::Component::Server::TCP does a lot of work in its constructor.
 The design goal is to push as much overhead into one-time construction
-so that ongoing run-time has less overhead.  Because of this, the
+so that ongoing runtime has less overhead.  Because of this, the
 server's constructor can take quite a daunting number of parameters.
 
 POE::Component::Server::TCP always returns a POE::Session ID for the
@@ -823,7 +791,7 @@ session that will be listening for new connections.
 Many of the constructor parameters have been previously described.
 They are covered briefly again below.
 
-=head3 Server Session Configuration
+=head3 Server Sesson Configuration
 
 These constructor parameters affect POE::Component::Server::TCP's main
 listening session.
@@ -839,7 +807,7 @@ C<SuccessEvent>.
 
 C<Acceptor> is optional; the default handler will create a new session
 for each connection.  All the "Client" constructor parameters are used
-to customize this session.  In other words, C<ClientInput> and such
+to customize this session.  In other words, C<CleintInput> and such
 B<are not used when C<Acceptor> is set>.
 
 The default C<Acceptor> adds significant convenience and flexibility
@@ -895,7 +863,7 @@ connections.  This may be useful if a server must perform lengthy
 initialization before allowing connections.  When the initialization
 finishes, it can yield(set_concurrency => -1) to enable connections.
 Likewise, a running server may yield(set_concurrency => 0) or any
-other number to dynamically tune its concurrency.  See L</EVENTS> for
+other number to dynamically tune its concurrency.  See L<EVENTS> for
 more about the set_concurrency event.
 
 Note: For C<Concurrency> to work with a custom C<Acceptor>, the
@@ -932,9 +900,8 @@ C<Domain> sets the address or protocol family within which to operate.
 The C<Domain> may be any value that POE::Wheel::SocketFactory
 supports.  AF_INET (Internet address space) is used by default.
 
-Use AF_INET6 for IPv6 support.  This constant is exported by Socket.
-Also be sure to have Socket::GetAddrInfo installed, which is required
-by POE::Wheel::SocketFactory for IPv6 support.
+Use AF_INET6 for IPv6 support.  This constant is exported by Socket6,
+which must be loaded B<before> POE::Component::Server::TCP.
 
 =head4 Error
 
@@ -958,8 +925,8 @@ transactions.
 C<Hostname> is the optional non-packed name of the interface the TCP
 server will bind to.  The hostname will always be resolved via
 inet_aton() and so can either be a dotted quad or a name.  Name
-resolution is a one-time start-up action; there are no ongoing
-run-time penalties for using it.
+resolution is a one-time startup action; there are no ongoing runtime
+penalties for using it.
 
 C<Hostname> guarantees name resolution, where C<Address> does not.
 It's therefore preferred to use C<Hostname> in cases where resolution
@@ -971,7 +938,7 @@ C<InlineStates> is optional.  If specified, it must hold a hashref of
 named callbacks.  Its syntax is that of POE:Session->create()'s
 inline_states parameter.
 
-Remember: These InlineStates handlers will be added to the client
+Remember: These InlineStates handlers will be added to the client 
 sessions, not to the main listening session.  A yield() in the listener
 will not reach these handlers.
 
@@ -1050,10 +1017,10 @@ client's session is started and ready for operation.  Banners are
 often sent to the remote client from this callback.
 
 The @_[ARG0..$#_] parameters to C<ClientConnected> are a copy of the
-values in the C<ClientArgs> constructor parameter's array reference.
+values in the C<ClientArgs> constructor parameter's array referece.
 The other @_ members are standard for a POE::Session _start handler.
 
-C<ClientConnected> is called once per session start-up.  It will never
+C<ClientConnected> is called once per session startup.  It will never
 be called twice for the same connection.
 
   ClientConnected => sub {
@@ -1157,7 +1124,7 @@ flushed before finishing a client shutdown.
 C<ClientInput> defines a per-connection callback to handle client
 input.  This callback receives its parameters directly from
 POE::Wheel::ReadWrite's C<InputEvent>.  ARG0 contains the input
-record, the format of which is defined by C<ClientFilter> or
+recored, the format of which is defined by C<ClientFilter> or
 C<ClientInputFilter>.  ARG1 has the wheel's unique ID, and so on.
 Please see POE:Wheel::ReadWrite for an in-depth description of
 C<InputEvent>.
@@ -1240,7 +1207,7 @@ in its L<Alias> parameter.
 =head3 disconnected
 
 The "disconnected" event informs the TCP server that a connection was
-closed.  It is needed when using L</Concurrency> with an L</Acceptor>
+closed.  It is needed when using L<Concurrency> with an L<Acceptor>
 callback.  The custom Acceptor must provide its own disconnect
 notification so that the server's connection counting logic works.
 
@@ -1250,7 +1217,7 @@ server will never know when clients have disconnected.
 =head3 set_concurrency
 
 "set_concurrency" set the number of simultaneous connections the
-server will be willing to accept.  See L</Concurrency> for more
+server will be willing to accept.  See L<Concurrency> for more
 details.  "set_concurrency" must have one parameter: the new maximum
 connection count.
 
@@ -1334,7 +1301,7 @@ procedure.
 
 =head3 remote_ip
 
-$_[HEAP]{remote_ip} contains the remote client's numeric address in
+$_[HEAP]{remote_ip} contains the remote client's nuermic address in
 human-readable form.
 
 =head3 remote_port
@@ -1345,7 +1312,7 @@ in human-readable form.
 =head3 remote_addr
 
 $_[HEAP]{remote_addr} contains the remote client's packed socket
-address in computer-readable form.
+address in computer-readabe form.
 
 =head3 shutdown
 
