@@ -1,11 +1,7 @@
-# $Id: HTTPD.pm 2447 2009-02-17 05:04:43Z rcaputo $
-
 # Filter::HTTPD Copyright 1998 Artur Bergman <artur@vogon.se>.
-
 # Thanks go to Gisle Aas for his excellent HTTP::Daemon.  Some of the
 # get code was copied out if, unfortunately HTTP::Daemon is not easily
 # subclassed for POE because of the blocking nature.
-
 # 2001-07-27 RCC: This filter will not support the newer get_one()
 # interface.  It gets single things by default, and it does not
 # support filter switching.  If someone absolutely needs to switch to
@@ -17,17 +13,23 @@ use strict;
 use POE::Filter;
 
 use vars qw($VERSION @ISA);
-$VERSION = do {my($r)=(q$Revision: 2447 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = '1.020';
+# NOTE - Should be #.### (three decimal places)
 @ISA = qw(POE::Filter);
 
-sub BUFFER        () { 0 }
-sub TYPE          () { 1 }
-sub FINISH        () { 2 }
-sub HEADER        () { 3 }
-sub CLIENT_PROTO  () { 4 }
+sub BUFFER        () { 0 } # raw data buffer to build requests
+sub STATE         () { 1 } # built a full request
+sub REQUEST       () { 2 } # partial request being built
+sub CLIENT_PROTO  () { 3 } # client protoco version requested
+sub CONTENT_LEN   () { 4 } # expected content length
+sub CONTENT_ADDED () { 5 } # amount of content added to request
+
+sub ST_HEADERS    () { 0x01 } # waiting for complete header block
+sub ST_CONTENT    () { 0x02 } # waiting for complete body
 
 use Carp qw(croak);
-use HTTP::Status qw( status_message RC_BAD_REQUEST RC_OK RC_LENGTH_REQUIRED );
+use HTTP::Status qw( status_message RC_BAD_REQUEST RC_OK RC_LENGTH_REQUIRED 
+                                    RC_REQUEST_ENTITY_TOO_LARGE );
 use HTTP::Request ();
 use HTTP::Response ();
 use HTTP::Date qw(time2str);
@@ -40,156 +42,69 @@ my $HTTP_1_1 = _http_version("HTTP/1.1");
 
 sub new {
   my $type = shift;
-  my $self = [
-    '',     # BUFFER
-    0,      # TYPE
-    0,      # FINISH
-    undef,  # HEADER
-    undef,  # CLIENT_PROTO
-  ];
-  bless $self, $type;
-  $self;
+  return bless(
+    [
+      '',         # BUFFER
+      ST_HEADERS, # STATE
+      undef,      # REQUEST
+      undef,      # CLIENT_PROTO
+      0,          # CONTENT_LEN
+      0,          # CONTENT_ADDED
+    ],
+    $type
+  );
 }
 
 #------------------------------------------------------------------------------
 
 sub get_one_start {
-    my ($self, $stream) = @_;
-    return if ( $self->[FINISH] );
-    $stream = [ $stream ] unless ( ref( $stream ) );
-    $self->[BUFFER] .= join( '', @$stream );
+  my ($self, $stream) = @_;
+  $self->[BUFFER] .= join( '', @$stream );
 }
 
 sub get_one {
-    my ($self) = @_;
-    return ( $self->[FINISH] ) ? [] : $self->get( [] );
-}
-
-sub get {
-  my ($self, $stream) = @_;
+  my ($self) = @_;
 
   # Need to check lengths in octets, not characters.
   BEGIN { eval { require bytes } and bytes->import; }
 
-  # Why?
-  local($_);
+  # Waiting for a complete suite of headers.
+  if ($self->[STATE] & ST_HEADERS) {
+    # Strip leading whitespace.
+    $self->[BUFFER] =~ s/^\s+//;
 
-  # Sanity check.  "finish" is set when a request has completely
-  # arrived.  Subsequent get() calls on the same request should not
-  # happen.
-  # TODO Maybe this should return [] instead of dying?
+    # No blank line yet.  Side effect: Raw headers block is extracted
+    # from the input buffer.
+    return [] unless (
+      $self->[BUFFER] =~
+      s/^(\S.*?(?:\x0D\x0A?\x0D\x0A?|\x0A\x0D?\x0A\x0D?))//s
+    );
 
-  if ($self->[FINISH]) {
+    # Raw headers block from the input buffer.
+    my $rh = $1;
 
-    # This works around a request length vs. actual content length
-    # error.  Looks like some browsers (mozilla!) sometimes add on an
-    # extra newline?
-
-    # return [] unless @$stream and grep /\S/, @$stream;
-
-    my @dump;
-    my $offset = 0;
-    $stream = $self->[BUFFER].join("", @$stream);
-    while (length $stream) {
-      my $line = substr($stream, 0, 16);
-      substr($stream, 0, 16) = '';
-
-      my $hexdump  = unpack 'H*', $line;
-      $hexdump =~ s/(..)/$1 /g;
-
-      $line =~ tr[ -~][.]c;
-      push @dump, sprintf( "%04x %-47.47s - %s\n", $offset, $hexdump, $line );
-      $offset += 16;
+    # Parse the request line.
+    if ($rh !~ s/^(\w+)[ \t]+(\S+)(?:[ \t]+(HTTP\/\d+\.\d+))?[^\012]*\012//) {
+      return [
+        $self->_build_error(RC_BAD_REQUEST, "Request line parse failure. ($rh)")
+      ];
     }
 
-    return [
-      $self->_build_error(
-        RC_BAD_REQUEST,
-        "Did not want any more data.  Got this:" .
-        "<p><pre>" . join("", @dump) . "</pre></p>"
-      )
-    ];
-  }
+    # Create an HTTP::Request object from values in the request line.
+    my ($method, $request_path, $proto) = ($1, $2, ($3 || "HTTP/0.9"));
 
-  # Accumulate data in a framing buffer.
+    # Fix a double starting slash on the path.  It happens.
+    $request_path =~ s!^//+!/!;
 
-  $self->[BUFFER] .= join('', @$stream);
+    my $r = HTTP::Request->new($method, URI->new($request_path));
+    $r->protocol($proto);
+    $self->[CLIENT_PROTO] = $proto = _http_version($proto);
 
-  # If headers were already received, then the framing buffer is
-  # purely content.  Return nothing until content-length bytes are in
-  # the buffer, then return the entire request.
+    # Parse headers.
 
-  if ($self->[HEADER]) {
-    my $buf = $self->[BUFFER];
-    my $r   = $self->[HEADER];
-    my $cl  = $r->content_length() || length($buf) || 0;
-
-    # Some browsers (like MSIE 5.01) send extra CRLFs after the
-    # content.  Shame on them.  Now we need a special case to drop
-    # their extra crap.
-    #
-    # We use the first $cl octets of the buffer as the request
-    # content.  It's then stripped away.  Leading whitespace in
-    # whatever is left is also stripped away.  Any nonspace data left
-    # over will throw an error.
-    #
-    # Four-argument substr() would be ideal here, but it's a
-    # relatively recent development.
-    #
-    # PG- CGI.pm only reads Content-Length: bytes from STDIN.
-    if (length($buf) >= $cl) {
-      $r->content(substr($buf, 0, $cl));
-      $self->[BUFFER] = substr($buf, $cl);
-      $self->[BUFFER] =~ s/^\s+//;
-
-      # We are sending this back, so won't need it anymore.
-      $self->[HEADER] = undef;
-      $self->[FINISH]++;
-      return [$r];
-    }
-
-    #print "$cl wanted, got " . length($buf) . "\n";
-    return [];
-  }
-
-  # Headers aren't already received.  Short-circuit header parsing:
-  # don't return anything until we've received a blank line.
-
-  return [] unless(
-    $self->[BUFFER] =~ /(\x0D\x0A?\x0D\x0A?|\x0A\x0D?\x0A\x0D?)/s
-  );
-
-  # Copy the buffer for header parsing, and remove the header block
-  # from the content buffer.
-
-  my $buf = $self->[BUFFER];
-  $self->[BUFFER] =~ s/.*?(\x0D\x0A?\x0D\x0A?|\x0A\x0D?\x0A\x0D?)//s;
-
-  # Parse the request line.
-  if ($buf !~ s/^(\w+)[ \t]+(\S+)(?:[ \t]+(HTTP\/\d+\.\d+))?[^\012]*\012//) {
-    return [
-      $self->_build_error(RC_BAD_REQUEST, "Request line parse failure.")
-    ];
-  }
-  my $proto = $3 || "HTTP/0.9";
-
-  # Use the request line to create a request object.
-
-  my $method = $1;
-  my $req_path = $2;
-  $req_path =~ s/^[\/]{2,}/\//; # fix double slash starting path
-
-  my $r = HTTP::Request->new($method, URI->new($req_path));
-  $r->protocol($proto);
-  $self->[CLIENT_PROTO] = $proto = _http_version($proto);
-
-  # Add the raw request's headers to the request object we'll be
-  # returning.
-
-  if ($proto >= $HTTP_1_0) {
-    my ($key,$val);
-    HEADER: while ($buf =~ s/^([^\012]*)\012//) {
-      $_ = $1;
+    my ($key, $val);
+    HEADER: while ($rh =~ s/^([^\012]*)\012//) {
+      local $_ = $1;
       s/\015$//;
       if (/^([\w\-~]+)\s*:\s*(.*)/) {
         $r->push_header($key, $val) if $key;
@@ -202,77 +117,131 @@ sub get {
         last HEADER;
       }
     }
-    $r->push_header($key,$val) if($key);
-  }
 
-  $self->[HEADER] = $r;
+    $r->push_header($key, $val) if $key;
 
-  # If this is a GET or HEAD request, we won't be expecting a message
-  # body.  Finish up.
-  $method = uc $r->method();
-  if ($method eq 'GET' or $method eq 'HEAD') {
-    $self->[FINISH]++;
-    # We are sending this back, so won't need it anymore.
-    $self->[HEADER] = undef;
-    return [$r];
-  }
+    # We got a full set of headers.  Fall through to content if we
+    # have a content length.
 
-  # However, if it's any other type of request, check whether the
-  # entire content has already been received!  If so, add that to the
-  # request and we're done.  Otherwise we'll expect a subsequent get()
-  # call to finish things up.
-
-  #print "post:$buf:\END BUFFER\n";
-  #print length($buf)."-".$r->content_length()."\n";
-
-  my $cl = $r->content_length();
-  unless(defined $cl) {
-    if($self->[CLIENT_PROTO] == 9) {
-      return [
-        $self->_build_error(
-          RC_BAD_REQUEST,
-          "POST request detected in an HTTP 0.9 transaction. " .
-          "POST is not a valid HTTP 0.9 transaction type. " .
-          "Please verify your HTTP version and transaction content."
-        )
-      ];
+    my $cl = $r->content_length();
+    if( defined $cl ) {
+        $cl =~ s/\D.*$//;
+        $cl ||= 0;
     }
-    elsif ($method eq 'OPTIONS') {
-      $self->[FINISH]++;
-      # OPTIONS requests can have an optional content length
-      # See http://www.faqs.org/rfcs/rfc2616.html, section 9.2
-      $self->[HEADER] = undef;
-      return [$r];
+    my $ce = $r->content_encoding();
+    
+#   The presence of a message-body in a request is signaled by the
+#   inclusion of a Content-Length or Transfer-Encoding header field in
+#   the request's message-headers. A message-body MUST NOT be included in
+#   a request if the specification of the request method (section 5.1.1)
+#   does not allow sending an entity-body in requests. A server SHOULD
+#   read and forward a message-body on any request; if the request method
+#   does not include defined semantics for an entity-body, then the
+#   message-body SHOULD be ignored when handling the request.
+#   - RFC2616
+
+    unless( defined $cl || defined $ce ) {
+        # warn "No body";
+        $self->_reset();
+        return [ $r ];
     }
-    else {
-      return [
-        $self->_build_error(RC_LENGTH_REQUIRED, "No content length found.")
-      ];
+    
+    # PG- GET shouldn't have a body. But RFC2616 talks about Content-Length
+    # for HEAD.  And My reading of RFC2616 is that HEAD is the same as GET.
+    # So logically, GET can have a body.  And RFC2616 says we SHOULD ignore
+    # it.
+    #
+    # What's more, in apache 1.3.28, a body on a GET or HEAD is read
+    # and discarded.  See ap_discard_request_body() in http_protocol.c and
+    # default_handler() in http_core.c
+    #
+    # Neither Firefox 2.0 nor Lynx 2.8.5 set Content-Length on a GET
+
+#   For compatibility with HTTP/1.0 applications, HTTP/1.1 requests
+#   containing a message-body MUST include a valid Content-Length header
+#   field unless the server is known to be HTTP/1.1 compliant. If a
+#   request contains a message-body and a Content-Length is not given,
+#   the server SHOULD respond with 400 (bad request) if it cannot
+#   determine the length of the message, or with 411 (length required) if
+#   it wishes to insist on receiving a valid Content-Length.
+# - RFC2616 
+#
+# PG- This seems to imply that we can either detect the length (but how
+#     would one do that?) or require a Content-Length header.  We do the
+#     latter.
+# 
+# PG- Dispite all the above, I'm not fully sure this implements RFC2616
+#     properly.  There's something about transfer-coding that I don't fully
+#     understand.
+
+    if ( not $cl) {         
+      # assume a Content-Length of 0 is valid pre 1.1
+      if ($self->[CLIENT_PROTO] >= $HTTP_1_1 and not defined $cl) {
+        # We have Content-Encoding, but not Content-Length.
+        $r = $self->_build_error(RC_LENGTH_REQUIRED, 
+                                 "No content length found.",
+                                 $r);
+      }
+      $self->_reset();
+      return [ $r ];
     }
+
+    $self->[REQUEST] = $r;
+    $self->[CONTENT_LEN] = $cl;
+    $self->[STATE] = ST_CONTENT;
+    # Fall through to content.
   }
 
-  unless ($cl =~ /^\d+$/) {
-    return [
-      $self->_build_error(
-        RC_BAD_REQUEST,
-        "Content length contains non-digits."
-      )
-    ];
-  }
+  # Waiting for content.
+  if ($self->[STATE] & ST_CONTENT) {
+    my $r         = $self->[REQUEST];
+    my $cl_needed = $self->[CONTENT_LEN] - $self->[CONTENT_ADDED];
+    die "already got enough content ($cl_needed needed)" if $cl_needed < 1;
 
-  if (length($buf) >= $cl) {
-    $r->content(substr($buf, 0, $cl));
-    $self->[BUFFER] = substr($buf, $cl);
+    # Not enough content to complete the request.  Add it to the
+    # request content, and return an incomplete status.
+    if (length($self->[BUFFER]) < $cl_needed) {
+      $r->add_content($self->[BUFFER]);
+      $self->[CONTENT_ADDED] += length($self->[BUFFER]);
+      $self->[BUFFER] = "";
+      return [];
+    }
+
+    # Enough data.  Add it to the request content.
+    # PG- CGI.pm only reads Content-Length: bytes from STDIN.
+
+    # Four-argument substr() would be ideal here, but it's not
+    # entirely backward compatible.
+    $r->add_content(substr($self->[BUFFER], 0, $cl_needed));
+    substr($self->[BUFFER], 0, $cl_needed) = "";
+
+    # Some browsers (like MSIE 5.01) send extra CRLFs after the
+    # content.  Shame on them.
     $self->[BUFFER] =~ s/^\s+//;
-    $self->[FINISH]++;
-    # We are sending this back, so won't need it anymore.
-    $self->[HEADER] = undef;
-    return [$r];
+
+    # XXX Should we throw the body away on a GET or HEAD? Probably not.
+
+    # XXX Should we parse Multipart Types bodies?
+
+    # Prepare for the next request, and return this one.
+    $self->_reset();
+    return [ $r ];
   }
 
-  return [];
+  # What are we waiting for?
+  die "unknown state $self->[STATE]";
 }
 
+# Prepare for next request
+sub _reset
+{
+   my($self) = @_;
+   $self->[STATE] = ST_HEADERS;
+   @$self[REQUEST, CLIENT_PROTO]       = (undef, undef);
+   @$self[CONTENT_LEN, CONTENT_ADDED]  = (0, 0);
+}
+
+
 #------------------------------------------------------------------------------
 
 sub put {
@@ -284,6 +253,7 @@ sub put {
   # to send it to a client.  Here I've stolen HTTP::Response's
   # as_string's code and altered it to use network newlines so picky
   # browsers like lynx get what they expect.
+  # PG- $r->as_string( "\x0D\x0A" ); would accomplish the same thing, no?
 
   foreach (@$responses) {
     my $code           = $_->code;
@@ -305,9 +275,6 @@ sub put {
     push @raw, join("\x0D\x0A", @headers, "") . $_->content;
   }
 
-  # Allow next request after we're done sending the response.
-  $self->[FINISH]--;
-
   \@raw;
 }
 
@@ -336,7 +303,7 @@ sub _http_version {
 # content.
 
 sub _build_basic_response {
-  my ($self, $content, $content_type, $status) = @_;
+  my ($self, $content, $content_type, $status, $message) = @_;
 
   # Need to check lengths in octets, not characters.
   BEGIN { eval { require bytes } and bytes->import; }
@@ -344,7 +311,7 @@ sub _build_basic_response {
   $content_type ||= 'text/html';
   $status       ||= RC_OK;
 
-  my $response = HTTP::Response->new($status);
+  my $response = HTTP::Response->new($status, $message);
 
   $response->push_header( 'Content-Type', $content_type );
   $response->push_header( 'Content-Length', length($content) );
@@ -354,13 +321,13 @@ sub _build_basic_response {
 }
 
 sub _build_error {
-  my($self, $status, $details) = @_;
+  my($self, $status, $details, $req) = @_;
 
   $status  ||= RC_BAD_REQUEST;
   $details ||= '';
   my $message = status_message($status) || "Unknown Error";
 
-  return $self->_build_basic_response(
+  my $resp = $self->_build_basic_response(
     ( "<html>" .
       "<head>" .
       "<title>Error $status: $message</title>" .
@@ -372,8 +339,11 @@ sub _build_error {
       "</html>"
     ),
     "text/html",
-    $status
+    $status,
+    $message
   );
+  $resp->request( $req ) if $req;
+  return $resp;
 }
 
 1;
@@ -403,7 +373,12 @@ POE::Filter::HTTPD - parse simple HTTP requests, and serialize HTTP::Response
 
       # It's a response for the client if there was a problem.
       if ($request->isa("HTTP::Response")) {
-        $_[HEAP]{client}->put($request);
+        my $response = $request;
+
+        $request = $response->request;
+        warn "ERROR: ", $request->message if $request;
+
+        $_[HEAP]{client}->put($response);
         $_[KERNEL]->yield("shutdown");
         return;
       }
@@ -438,12 +413,16 @@ POE::Filter::HTTPD - parse simple HTTP requests, and serialize HTTP::Response
 
 =head1 DESCRIPTION
 
-POE::Filter::HTTPD interprets input streams as HTTP 0.9 or 1.0
+POE::Filter::HTTPD interprets input streams as HTTP 0.9, 1.0 or 1.1
 requests.  It returns a HTTP::Request objects upon successfully
-parsing a request.  On failure, it returns an HTTP::Response object
-describing the failure.  The intention is that application code will
-notice the HTTP::Response and send it back without further processing.
-This is illustrated in the L</SYNOPSIS>.
+parsing a request.  
+
+On failure, it returns an HTTP::Response object describing the
+failure.  The intention is that application code will notice the
+HTTP::Response and send it back without further processing. The
+erroneous request object is sometimes available via the
+L<HTTP::Response/request> method.  This is illustrated in the
+L</SYNOPSIS>.
 
 For output, POE::Filter::HTTPD accepts HTTP::Response objects and
 returns their corresponding streams.
@@ -477,6 +456,17 @@ follows this convention.  In the transaction detailed above, the
 Filter::HTTPD based daemon will return a 400 error since POST is not a
 valid HTTP/0.9 request type.
 
+Upon handling a request error, it is most expedient and reliable to
+respond with the error and shut down the connection.  Invalid HTTP
+requests may corrupt the request stream.  For example, the absence of
+a Content-Length header signals that a request has no content.
+Requests with content but not that header will be broken into a
+content-less request and invalid data.  The invalid data may also
+appear to be a request!  Hilarity will ensue, possibly repeatedly,
+until the filter can find the next valid request.  By shutting down
+the connection on the first sign of error, the client can retry its
+request with a clean connection and filter.
+
 =head1 Streaming Media
 
 It is possible to use POE::Filter::HTTPD for streaming content, but an
