@@ -1,11 +1,9 @@
-# $Id: Loop.pm 2357 2008-06-20 17:41:54Z rcaputo $
-
 package POE::Loop;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2357 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = '1.020'; # NOTE - Should be #.### (three decimal places)
 
 1;
 
@@ -82,7 +80,7 @@ Here is a skeleton of a POE::Loop subclass:
   package POE::Loop::YourToolkit;
 
   use vars qw($VERSION);
-  $VERSION = do {my@r=(q$Revision: 2357 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
+  $VERSION = '1.000'; # NOTE - Should be #.### (three decimal places)
 
   package POE::Kernel;
 
@@ -317,7 +315,7 @@ Here's Gtk's:
           $self->_dispatch_event(
             $self, $self,
             EN_SIGNAL, ET_SIGNAL, [ 'UIDESTROY' ],
-            __FILE__, __LINE__, time(), -__LINE__
+            __FILE__, __LINE__, undef, time(), -__LINE__
           );
         }
         return 0;
@@ -571,3 +569,4 @@ and POE's licensing.
 =cut
 
 # rocco // vim: ts=2 sw=2 expandtab
+# TODO - Edit.
