@@ -1,5 +1,3 @@
-# $Id: PerlSignals.pm 2307 2008-03-31 11:46:07Z nothingmuch $
-
 # Plain Perl signal handling is something shared by several event
 # loops.  The invariant code has moved out here so that each loop may
 # use it without reinventing it.  This will save maintenance and
@@ -10,7 +8,7 @@ package POE::Loop::PerlSignals;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2307 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = '1.020'; # NOTE - Should be #.### (three decimal places)
 
 # Everything plugs into POE::Kernel.
 package POE::Kernel;
@@ -26,6 +24,15 @@ my %signal_watched;
 # Signal handlers/callbacks.
 
 sub _loop_signal_handler_generic {
+  if( USE_SIGNAL_PIPE ) {
+    POE::Kernel->_data_sig_pipe_send( $_[0] );
+  }
+  else {
+    _loop_signal_handler_generic_bottom( $_[0] );
+  }
+}
+
+sub _loop_signal_handler_generic_bottom {
   if (TRACE_SIGNALS) {
     POE::Kernel::_warn "<sg> Enqueuing generic SIG$_[0] event";
   }
@@ -37,7 +44,17 @@ sub _loop_signal_handler_generic {
   $SIG{$_[0]} = \&_loop_signal_handler_generic;
 }
 
+##
 sub _loop_signal_handler_pipe {
+  if( USE_SIGNAL_PIPE ) {
+    POE::Kernel->_data_sig_pipe_send( $_[0] );
+  }
+  else {
+    _loop_signal_handler_pipe_bottom( $_[0] );
+  }
+}
+
+sub _loop_signal_handler_pipe_bottom {
   if (TRACE_SIGNALS) {
     POE::Kernel::_warn "<sg> Enqueuing PIPE-like SIG$_[0] event";
   }
@@ -49,13 +66,27 @@ sub _loop_signal_handler_pipe {
   $SIG{$_[0]} = \&_loop_signal_handler_pipe;
 }
 
-# only used under USE_SIGCHLD
+## only used under USE_SIGCHLD
 sub _loop_signal_handler_chld {
+  if( USE_SIGNAL_PIPE ) {
+    POE::Kernel->_data_sig_pipe_send( 'CHLD' );
+  }
+  else {
+    _loop_signal_handler_chld_bottom( $_[0] );
+  }
+}
+
+sub _loop_signal_handler_chld_bottom {
   if (TRACE_SIGNALS) {
     POE::Kernel::_warn "<sg> Enqueuing CHLD-like SIG$_[0] event";
   }
 
   $poe_kernel->_data_sig_enqueue_poll_event();
+
+  # TODO - Per https://rt.cpan.org/Ticket/Display.html?id=45109
+  # resetting %SIG may need to be deferred until after child processes
+  # have been reaped.  Waiting for confirmation that the recent
+  # USE_SIGNALS changes have not fixed deep recursion on HP-UX.
   $SIG{$_[0]} = \&_loop_signal_handler_chld;
 }
 
@@ -101,8 +132,17 @@ sub loop_ignore_signal {
 
   delete $signal_watched{$signal};
 
-  unless ( USE_SIGCHLD ) {
     if ($signal eq 'CHLD' or $signal eq 'CLD') {
+    if ( USE_SIGCHLD ) {
+      if( $self->_data_sig_child_procs) {
+        # We need SIGCHLD to stay around after shutdown, so that
+        # child processes may be reaped and kr_child_procs=0
+        if (TRACE_SIGNALS) {
+          POE::Kernel::_warn "<sg> Keeping SIG$signal anyway!";
+        }
+        return;
+      }
+    } else {
       $self->_data_sig_cease_polling();
       # We should never twiddle $SIG{CH?LD} under poe, unless we want to
       # override system() and friends. --hachi
@@ -111,12 +151,17 @@ sub loop_ignore_signal {
     }
   }
 
+  delete $signal_watched{$signal};
+
+  my $state = 'DEFAULT';
   if ($signal eq 'PIPE') {
-    $SIG{$signal} = "IGNORE";
-    return;
+    $state = "IGNORE";
   }
 
-  $SIG{$signal} = "DEFAULT";
+  if (TRACE_SIGNALS) {
+    POE::Kernel::_warn "<sg> $state SIG$signal";
+  }
+  $SIG{$signal} = $state;
 }
 
 sub loop_ignore_all_signals {
@@ -132,7 +177,7 @@ __END__
 
 =head1 NAME
 
-POE::Loop::PerlSignals - plain Perl signal handlers used by many loops
+POE::Loop::PerlSignals - common signal handling routines for POE::Loop bridges
 
 =head1 SYNOPSIS
 
@@ -140,10 +185,12 @@ See L<POE::Loop>.
 
 =head1 DESCRIPTION
 
-This class is an implementation of the signal handling functions
-defined by the abstract POE::Loop interface.  It follows POE::Loop's
-public interface for signal handling exactly.  Therefore, please see
-L<POE::Loop> for its documentation.
+POE::Loop::PerlSignals implements common code to handle signals for
+many different event loops.  Most loops don't handle signals natively,
+so this code has been abstracted into a reusable mix-in module.
+
+POE::Loop::PerlSignals follows POE::Loop's public interface for signal
+handling.  Therefore, please see L<POE::Loop> for more details.
 
 =head1 SEE ALSO
 
@@ -157,4 +204,4 @@ and POE's licensing.
 =cut
 
 # rocco // vim: ts=2 sw=2 expandtab
-# TODO - Redocument.
+# TODO - Edit.
