@@ -1,11 +1,9 @@
-# $Id: Sessions.pm 2312 2008-04-19 05:49:16Z rcaputo $
-
 # Manage session data structures on behalf of POE::Kernel.
 
 package POE::Resource::Sessions;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2312 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = '1.020'; # NOTE - Should be #.### (three decimal places)
 
 # These methods are folded into POE::Kernel;
 package POE::Kernel;
@@ -176,6 +174,10 @@ sub _data_ses_free {
   $self->_data_handle_clear_session($session); # Remove all leftover handles.
   $self->_data_ev_clear_session($session);     # Remove all leftover events.
 
+  if (TRACE_PROFILE) {
+    $self->_data_stat_clear_session($session);
+  }
+
   # Remove the session itself.
 
   delete $kr_sessions{$session};
@@ -472,13 +474,13 @@ sub _data_ses_stop {
     $self->_dispatch_event(
       $parent, $self,
       EN_CHILD, ET_CHILD, [ CHILD_GAIN, $child ],
-      __FILE__, __LINE__, time(), -__LINE__
+      __FILE__, __LINE__, undef, time(), -__LINE__
     );
     $self->_dispatch_event(
       $child, $self,
       EN_PARENT, ET_PARENT,
       [ $self->_data_ses_get_parent($child), $parent, ],
-      __FILE__, __LINE__, time(), -__LINE__
+      __FILE__, __LINE__, undef, time(), -__LINE__
     );
   }
 
@@ -487,7 +489,7 @@ sub _data_ses_stop {
   my $stop_return = $self->_dispatch_event(
     $session, $self->get_active_session(),
     EN_STOP, ET_STOP, [],
-    __FILE__, __LINE__, time(), -__LINE__
+    __FILE__, __LINE__, undef, time(), -__LINE__
   );
 
   # If the departing session has a parent, notify it that the session
@@ -497,7 +499,7 @@ sub _data_ses_stop {
     $self->_dispatch_event(
       $parent, $self,
       EN_CHILD, ET_CHILD, [ CHILD_LOSE, $session, $stop_return ],
-      __FILE__, __LINE__, time(), -__LINE__
+      __FILE__, __LINE__, undef, time(), -__LINE__
     );
   }
 
@@ -524,26 +526,46 @@ __END__
 
 =head1 NAME
 
-POE::Resource::Sessions - manage session data structures for POE::Kernel
+POE::Resource::Sessions - internal session manager for POE::Kernel
 
 =head1 SYNOPSIS
 
-Used internally by POE::Kernel.  Better documentation will be
-forthcoming.
+There is no public API.
 
 =head1 DESCRIPTION
 
-This module encapsulates and provides accessors for POE::Kernel's data
-structures that manage sessions themselves.  It is used internally by
-POE::Kernel and has no public interface.
+POE::Resource::Sessions is a mix-in class for POE::Kernel.  It
+provides the internal features that manage sessions, regardless of the
+session type.  It is used internally by POE::Kernel. so it has no
+public interface.
 
 =head1 SEE ALSO
 
-See L<POE::Kernel> and L<POE::Session> for documentation on sessions.
+See L<POE::Session> and L<POE::NFA> for one type of session.  CPAN
+also have others.
+
+See L<POE::Kernel/Sessions> for a discussion about POE::Kernel
+sessions.
+
+See L<POE::Kernel/Session Lifespans> to learn why sessions run, and
+how to stop them.
+
+See L<POE::Kernel/Session Management> for information about managing
+sessions in your applications, and the events that occur when sessions
+come and go.
+
+See L<POE::Kernel/Session Helper Methods> for friend methods between
+POE::Kernel and POE::Session classes.
+
+See L<POE::Kernel/Resources> for for public information about POE
+resources.
+
+See L<POE::Resource> for general discussion about resources and the
+classes that manage them.
 
 =head1 BUGS
 
-Probably.
+None known.
 
 =head1 AUTHORS & COPYRIGHTS
 
@@ -552,4 +574,4 @@ Please see L<POE> for more information about authors and contributors.
 =cut
 
 # rocco // vim: ts=2 sw=2 expandtab
-# TODO - Redocument.
+# TODO - Edit.
