@@ -1,11 +1,9 @@
-# $Id: Session.pm 2262 2007-12-12 03:18:11Z gwyn17 $
-
 package POE::Session;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2262 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = '1.020'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw(carp croak);
 use Errno;
@@ -41,7 +39,7 @@ sub _define_assert {
   no strict 'refs';
   foreach my $name (@_) {
 
-    BEGIN { $^W = 0 };
+    local $^W = 0;
 
     next if defined *{"ASSERT_$name"}{CODE};
     if (defined *{"POE::Kernel::ASSERT_$name"}{CODE}) {
@@ -63,7 +61,7 @@ sub _define_assert {
 sub _define_trace {
   no strict 'refs';
 
-  BEGIN { $^W = 0 };
+  local $^W = 0;
 
   foreach my $name (@_) {
     next if defined *{"TRACE_$name"}{CODE};
@@ -671,6 +669,7 @@ sub get_heap {
 # steering me right on this one.
 
 my %anonevent_parent_id;
+my %anonevent_weakened;
 
 # I assume that when the postback owner loses all reference to it,
 # they are done posting things back to us.  That's when the postback's
@@ -679,7 +678,19 @@ my %anonevent_parent_id;
 sub POE::Session::AnonEvent::DESTROY {
   my $self = shift;
   my $parent_id = delete $anonevent_parent_id{$self};
-  $POE::Kernel::poe_kernel->refcount_decrement( $parent_id, 'anon_event' );
+  unless (delete $anonevent_weakened{$self}) {
+    $POE::Kernel::poe_kernel->refcount_decrement( $parent_id, 'anon_event' );
+  }
+}
+
+sub POE::Session::AnonEvent::weaken {
+  my $self = shift;
+  unless ($anonevent_weakened{$self}) {
+    my $parent_id = $anonevent_parent_id{$self};
+    $POE::Kernel::poe_kernel->refcount_decrement( $parent_id, 'anon_event' );
+    $anonevent_weakened{$self} = 1;
+  }
+  return $self;
 }
 
 # Tune postbacks depending on variations in toolkit behavior.
@@ -742,7 +753,6 @@ sub callback {
   return $callback;
 }
 
-###############################################################################
 1;
 
 __END__
@@ -1062,12 +1072,12 @@ what the heck---go ahead and mix in some inline code as well.
 However only one handler may be assigned to a given event name.
 Duplicates will overwrite earlier ones.
 
-event_1a is handled by calling C<$object_1->method_1a(...)>.  C<$_[OBJECT]>
+event_1a is handled by calling C<< $object_1->method_1a(...) >>.  C<$_[OBJECT]>
 is C<$object_1> in this case.  C<$_[HEAP]> belongs to the session, which
 means anything stored there will be available to any other event
 handler regardless of the object.
 
-event_2a is handled by calling C<$object_2->method_2a(...)>.  In this
+event_2a is handled by calling C<< $object_2->method_2a(...) >>.  In this
 case CL</$_[OBJECT]> is $object_2.  C<$_[HEAP]> is the same anonymous hashref
 that was passed to the event_1a handler, though.  The methods are resolved
 when the event is handled (late-binding).
@@ -1087,8 +1097,8 @@ single object can't handle events from more than one session:
     );
   }
 
-Now C<$object_4->method_4(...)> may be called to handle events from one
-of two sessions.  In both cases, C<$_[OBJECT]> will be C<$object_4>, but
+Now C<< $object_4->method_4(...) >> may be called to handle events from one of
+two sessions.  In both cases, C<$_[OBJECT]> will be C<$object_4>, but
 C<$_[HEAP]> will hold data for a particular session.
 
 The same goes for inline states.  One subroutine may handle events
@@ -1203,11 +1213,11 @@ Be very careful with closures, however.  L</Beware circular references>.
 
 C<object_states> associates one or more objects to a session and maps
 event names to the object methods that will handle them.  It's value
-is an CB<ARRAYREF>; C<HASHREFs> would stringify the objects, ruining them
+is an C<ARRAYREF>; C<HASHREFs> would stringify the objects, ruining them
 for method invocation.
 
-Here _start is handled by C<$object->_session_start()> and _stop triggers
-C<$object->_session_stop()>:
+Here _start is handled by C<< $object->_session_start() >> and _stop triggers
+C<< $object->_session_stop() >>:
 
   POE::Session->create(
     object_states => [
@@ -1269,11 +1279,11 @@ The value for C<package_states> is an B<ARRAYREF> to be consistent
 with C<object_states>, even though class names (also known as package names) are
 already strings, so it's not necessary to avoid stringifying them.
 
-Here _start is handled by C<$class_name->_session_start()> and _stop
-triggers C<$class_name->_session_stop()>:
+Here _start is handled by C<< $class_name->_session_start() >> and _stop
+triggers C<< $class_name->_session_stop() >>:
 
   POE::Session->create(
-    class_states => [
+    package_states => [
       $class_name => {
         _start => '_session_start',
         _stop  => '_session_stop',
@@ -1282,11 +1292,11 @@ triggers C<$class_name->_session_stop()>:
   );
 
 POE::Session also supports a short form where the event and method
-names are identical.  Here _start invokes C<$class_name->_start()>, and
-_stop triggers C<$class_name->_stop()>:
+names are identical.  Here _start invokes C<< $class_name->_start() >>, and
+_stop triggers C<< $class_name->_stop() >>:
 
   POE::Session->create(
-    class_states => [
+    package_states => [
       $class_name => [ '_start', '_stop' ],
     ]
   );
@@ -1305,8 +1315,8 @@ to store 4.29 billion session objects.
 
 A session's ID is unique within a running process, but multiple
 processes are likely to have the same session IDs.  If a global ID is
-required, it will need to include both C<$_[KERNEL]->ID> and
-C<$_[SESSION]->ID>.
+required, it will need to include both C<< $_[KERNEL]->ID >> and
+C<< $_[SESSION]->ID >>.
 
 =head2 option OPTION_NAME [, OPTION_VALUE [, OPTION_NAME, OPTION_VALUE]... ]
 
@@ -1420,9 +1430,17 @@ handle_ok_button():
   Postback created with (8 6 7).
   Postback called with (5 3 0 9).
 
-
 Postbacks hold references to their target sessions.  Therefore
-sessions with outstanding postbacks will remain active.
+sessions with outstanding postbacks will remain active.  Under every
+event loop except Tk, postbacks are blessed so that DESTROY may be
+called when their users are done.  This triggers a decrement on their
+reference counts, allowing sessions to stop.
+
+Postbacks have one method, weaken(), which may be used to reduce their
+reference counts upon demand.  weaken() returns the postback, so you
+can do:
+
+  my $postback = $session->postback("foo")->weaken();
 
 Postbacks were created as a thin adapter between callback libraries
 and POE.  The problem at hand was how to turn callbacks from the Tk
@@ -1447,8 +1465,8 @@ passed by the Gtk2 button will be in ARG1.
 
 =head2 callback EVENT_NAME, EVENT_PARAMETERS
 
-callback() manufactures callbacks that use C<$poe_kernel->call()> to
-deliver POE events rather than C<$poe_kernel->post()>.  It is identical
+callback() manufactures callbacks that use C<< $poe_kernel->call() >> to
+deliver POE events rather than C<< $poe_kernel->post() >>.  It is identical
 to C<postback()> in every other respect.
 
 callback() was created to avoid race conditions that arise when
@@ -1541,7 +1559,7 @@ that are passed to the session's _start handler.
     return $self->SUPER::try_alloc(@args);
   }
 
-=head1 POE::Session'S EVENTS
+=head1 POE::Session's EVENTS
 
 Please do not define new events that begin with a leading underscore.
 POE claims /^_/ events as its own.
@@ -1624,9 +1642,18 @@ value wraps.  This can occur after as I<few> as 4.29 billion sessions.
 
 =head2 Beware circular references
 
-The following creates a circular reference:
+As you're probably aware, a circular reference is when a variable is
+part of a reference chain that eventually refers back to itself.  Perl
+will not reclaim the memory involved in such a reference chain until
+the chain is manually broken.
 
-  my $session = POE::Session->create(
+Here a POE::Session is created that refers to itself via an external
+scalar.  The event handlers import $session via closures which are in
+turn stored within $session.  Even if this session stops, the circular
+references will remain.
+
+  my $session;
+  $session = POE::Session->create(
     inline_states => {
       _start => sub {
         $_[HEAP]->{todo} = [ qw( step1 step2 step2a ) ],
@@ -1641,33 +1668,81 @@ The following creates a circular reference:
     }
   );
 
-Note also that a anonymous sub creates a closure on all lexical variables in 
-the scope it was defined in, even if it doesn't reference them:
+Reduced to its essence:
+
+  my %event_handlers;
+  $event_handler{_start} = sub { \%event_handlers };
+
+Note also that a anonymous sub creates a closure on all lexical
+variables in the scope it was defined in, even if it doesn't reference
+them.  $session is still being held in a circular reference here:
 
   my $self = $package->new;
-  my $session = POE::Session->create(
+  my $session;
+  $session = POE::Session->create(
     inline_state => {
       _start => sub { $self->_start( @_[ARG0..$#_] ) }
     }
   );
 
-To avoid this, must hold onto a session's ID, rather then the session's
-object.  You may convert a session ID back into the session's object with
-L<POE::Kernel->session_id_to_session()|POE::Kernel/session_id_to_session>.
+To avoid this, a session may set an alias for itself.  Other parts of
+the program may then refer to it by alias.  In this case, one needn't
+keep track of the session themselves (POE::Kernel will do it anyway).
 
-  my $session = POE::Session->create(
+  POE::Session->create(
     inline_states => {
       _start => sub {
-        $session = $session->ID;
         $_[HEAP]->{todo} = [ qw( step1 step2 step2a ) ],
-        $_[KERNEL]->post( $session, 'next' );
+        $_[KERNEL]->alias_set('step_doer');
+        $_[KERNEL]->post( 'step_doer', 'next' );
       },
+      next => sub {
+        my $next = shift @{ $_[HEAP]->{todo} };
+        return unless $next;
+        $_[KERNEL]->post( 'step_doer', $next );
+      }
       # ....
     }
   );
 
+Aliases aren't even needed in the previous example because the session
+refers to itself.  One could instead use POE::Kernel's yield() method
+to post the event back to the current session:
 
+  next => sub {
+    my $next = shift @{ $_[HEAP]->{todo} };
+    return unless $next;
+    $_[KERNEL]->yield( $next );
+  }
 
+Or the L<$_[SESSION]> parameter passed to every event handler, but
+yield() is more efficient.
+
+  next => sub {
+    my $next = shift @{ $_[HEAP]->{todo} };
+    return unless $next;
+    $_[KERNEL]->post( $_[SESSION], $next );
+  }
+
+Along the same lines as L<$_[SESSION]>, a session can respond back to
+the sender of an event by posting to L<$_[SENDER]>.  This is great for
+responding to requests.
+
+If a program must hold onto some kind of dynamic session reference,
+it's recommended to use the session's numeric ID rather than the
+object itself.  A session ID may be converted back into its object,
+but post() accepts session IDs as well as objects and aliases:
+
+  my $session_id;
+  $session_id = POE::Session->create(
+    inline_states => {
+      _start => sub {
+        $_[HEAP]->{todo} = [ qw( step1 step2 step2a ) ],
+        $_[KERNEL]->post( $session_id, 'next' );
+      },
+      # ....
+    }
+  )->ID;
 
 =head1 AUTHORS & COPYRIGHTS
 
@@ -1676,3 +1751,4 @@ Please see L<POE> for more information about authors and contributors.
 =cut
 
 # rocco // vim: ts=2 sw=2 expandtab
+# TODO - Edit.
